/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.common;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.common.IResultType;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.collision.CollisionCalcFacet;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;

/**
 * Processor to generate the facet and data series of calculated bed collision counts
 *
 * @author Matthias Schäfer
 *
 */
public final class CollisionCalcProcessor extends AbstractSInfoProcessor {

    public static final String FACET_COLLISION_CALC_COUNT = "sinfo_facet_collision_calc_count";

    private static final String I18N_AXIS_LABEL = "chart.collision_count.section.yaxis.label";

    private static final String I18N_SERIES_NAME_PATTERN = "collision.count.title";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_COLLISION_CALC_COUNT);
    }

    public CollisionCalcProcessor() {
        super(I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        return buildSeriesForType(generator, bundle, theme, visible, SInfoResultType.collisionCount);
    }

    // FIXME: move to super classs and use in many implementations
    protected final String buildSeriesForType(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible,
            final IResultType resultType) {
        final CallContext context = generator.getContext();
        final Map<String, String> metaData = bundle.getFacet().getMetaData();

        final Artifact artifact = bundle.getArtifact();

        final StyledXYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);
        series.putMetaData(metaData, artifact, context);

        final String facetName = bundle.getFacetName();

        final AbstractCalculationResult data = (AbstractCalculationResult) bundle.getData(context);
        if (data == null) {
            // Check has been here before so we keep it for security reasons
            // this should never happen though.
            throw new IllegalStateException("Data is null for facet: " + facetName);
        }

        final double[][] points = data.getStationPoints(resultType);

        StyledSeriesBuilder.addPoints(series, points, true);
        generator.addAxisSeries(series, getAxisName(), visible);

        return metaData.get("Y");
    }

    public static final Facet createFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int index) {
        final String facetDescription = Resources.getMsg(context.getMeta(), I18N_SERIES_NAME_PATTERN, I18N_SERIES_NAME_PATTERN, result.getLabel());
        return new CollisionCalcFacet(index, FACET_COLLISION_CALC_COUNT, facetDescription, I18N_AXIS_LABEL, ComputeType.ADVANCE, id, hash);
    }
}